@@ -29,6 +29,7 @@ from ansible.utils.plugins import *
 from ansible.utils import template
 from ansible.callbacks import display
 import ansible.constants as C
+import ast
 import time
 import StringIO
 import stat
@@ -43,6 +44,9 @@ import getpass
 import sys
 import textwrap
 
+#import vault
+from vault import VaultLib
+
 VERBOSITY=0
 
 # list of all deprecation messages to prevent duplicate display
@@ -87,19 +91,29 @@ def key_for_hostname(hostname):
     if not KEYCZAR_AVAILABLE:
         raise errors.AnsibleError("python-keyczar must be installed on the control machine to use accelerated modes")
 
-    key_path = os.path.expanduser("~/.fireball.keys")
+    key_path = os.path.expanduser(C.ACCELERATE_KEYS_DIR)
     if not os.path.exists(key_path):
-        os.makedirs(key_path)
-    key_path = os.path.expanduser("~/.fireball.keys/%s" % hostname)
+        os.makedirs(key_path, mode=0700)
+        os.chmod(key_path, int(C.ACCELERATE_KEYS_DIR_PERMS, 8))
+    elif not os.path.isdir(key_path):
+        raise errors.AnsibleError('ACCELERATE_KEYS_DIR is not a directory.')
+
+    if stat.S_IMODE(os.stat(key_path).st_mode) != int(C.ACCELERATE_KEYS_DIR_PERMS, 8):
+        raise errors.AnsibleError('Incorrect permissions on ACCELERATE_KEYS_DIR (%s)' % (C.ACCELERATE_KEYS_DIR,))
+
+    key_path = os.path.join(key_path, hostname)
 
     # use new AES keys every 2 hours, which means fireball must not allow running for longer either
     if not os.path.exists(key_path) or (time.time() - os.path.getmtime(key_path) > 60*60*2):
         key = AesKey.Generate()
-        fh = open(key_path, "w")
+        fd = os.open(key_path, os.O_WRONLY | os.O_CREAT, int(C.ACCELERATE_KEYS_FILE_PERMS, 8))
+        fh = os.fdopen(fd, 'w')
         fh.write(str(key))
         fh.close()
         return key
     else:
+        if stat.S_IMODE(os.stat(key_path).st_mode) != int(C.ACCELERATE_KEYS_FILE_PERMS, 8):
+            raise errors.AnsibleError('Incorrect permissions on ACCELERATE_KEYS_FILE (%s)' % (key_path,))
         fh = open(key_path)
         key = AesKey.Read(fh.read())
         fh.close()
@@ -163,58 +177,47 @@ def is_changed(result):
 
     return (result.get('changed', False) in [ True, 'True', 'true'])
 
-def check_conditional(conditional, basedir, inject, fail_on_undefined=False, jinja2=False):
+def check_conditional(conditional, basedir, inject, fail_on_undefined=False):
+
+    if conditional is None or conditional == '':
+        return True
 
     if isinstance(conditional, list):
         for x in conditional:
-            if not check_conditional(x, basedir, inject, fail_on_undefined=fail_on_undefined, jinja2=jinja2):
+            if not check_conditional(x, basedir, inject, fail_on_undefined=fail_on_undefined):
                 return False
         return True
 
-    if jinja2:
-        conditional = "jinja2_compare %s" % conditional
-
-    if conditional.startswith("jinja2_compare"):
-        conditional = conditional.replace("jinja2_compare ","")
-        # allow variable names
-        if conditional in inject and str(inject[conditional]).find('-') == -1:
-            conditional = inject[conditional]
-        conditional = template.template(basedir, conditional, inject, fail_on_undefined=fail_on_undefined)
-        original = str(conditional).replace("jinja2_compare ","")
-        # a Jinja2 evaluation that results in something Python can eval!
-        presented = "{%% if %s %%} True {%% else %%} False {%% endif %%}" % conditional
-        conditional = template.template(basedir, presented, inject)
-        val = conditional.strip()
-        if val == presented:
-            # the templating failed, meaning most likely a 
-            # variable was undefined. If we happened to be 
-            # looking for an undefined variable, return True,
-            # otherwise fail
-            if conditional.find("is undefined") != -1:
-                return True
-            elif conditional.find("is defined") != -1:
-                return False
-            else:
-                raise errors.AnsibleError("error while evaluating conditional: %s" % original)
-        elif val == "True":
-            return True
-        elif val == "False":
-            return False
-        else:
-            raise errors.AnsibleError("unable to evaluate conditional: %s" % original)
-
     if not isinstance(conditional, basestring):
         return conditional
 
-    try:
-        conditional = conditional.replace("\n", "\\n")
-        result = safe_eval(conditional)
-        if result not in [ True, False ]:
-            raise errors.AnsibleError("Conditional expression must evaluate to True or False: %s" % conditional)
-        return result
-
-    except (NameError, SyntaxError):
-        raise errors.AnsibleError("Could not evaluate the expression: (%s)" % conditional)
+    conditional = conditional.replace("jinja2_compare ","")
+    # allow variable names
+    if conditional in inject and str(inject[conditional]).find('-') == -1:
+        conditional = inject[conditional]
+    conditional = template.template(basedir, conditional, inject, fail_on_undefined=fail_on_undefined)
+    original = str(conditional).replace("jinja2_compare ","")
+    # a Jinja2 evaluation that results in something Python can eval!
+    presented = "{%% if %s %%} True {%% else %%} False {%% endif %%}" % conditional
+    conditional = template.template(basedir, presented, inject)
+    val = conditional.strip()
+    if val == presented:
+        # the templating failed, meaning most likely a 
+        # variable was undefined. If we happened to be 
+        # looking for an undefined variable, return True,
+        # otherwise fail
+        if conditional.find("is undefined") != -1:
+            return True
+        elif conditional.find("is defined") != -1:
+            return False
+        else:
+            raise errors.AnsibleError("error while evaluating conditional: %s" % original)
+    elif val == "True":
+        return True
+    elif val == "False":
+        return False
+    else:
+        raise errors.AnsibleError("unable to evaluate conditional: %s" % original)
 
 def is_executable(path):
     '''is the given path executable?'''
@@ -261,6 +264,8 @@ def path_dwim(basedir, given):
     elif given.startswith("~"):
         return os.path.abspath(os.path.expanduser(given))
     else:
+        if basedir is None:
+            basedir = "."
         return os.path.abspath(os.path.join(basedir, given))
 
 def path_dwim_relative(original, dirname, source, playbook_base, check=True):
@@ -371,7 +376,7 @@ It should be written as:
 """
         return msg
 
-    elif len(probline) and len(probline) >= column and probline[column] == ":" and probline.count(':') > 1:
+    elif len(probline) and len(probline) > 1 and len(probline) > column and probline[column] == ":" and probline.count(':') > 1:
         msg = msg + """
 This one looks easy to fix.  There seems to be an extra unquoted colon in the line 
 and this is confusing the parser. It was only expecting to find one free 
@@ -403,7 +408,7 @@ Or:
                 match = True
             elif middle.startswith('"') and not middle.endswith('"'):
                 match = True
-            if middle[0] in [ '"', "'" ] and middle[-1] in [ '"', "'" ] and probline.count("'") > 2 or probline.count("'") > 2:
+            if len(middle) > 0 and middle[0] in [ '"', "'" ] and middle[-1] in [ '"', "'" ] and probline.count("'") > 2 or probline.count("'") > 2:
                 unbalanced = True
             if match:
                 msg = msg + """
@@ -458,8 +463,30 @@ Note: The error may actually appear before this position: line %s, column %s
 %s
 %s""" % (path, mark.line + 1, mark.column + 1, before_probline, probline, arrow)
 
-        msg = process_common_errors(msg, probline, mark.column)
+        unquoted_var = None
+        if '{{' in probline and '}}' in probline:
+            if '"{{' not in probline or "'{{" not in probline:
+                unquoted_var = True
 
+        msg = process_common_errors(msg, probline, mark.column)
+        if not unquoted_var:
+            msg = process_common_errors(msg, probline, mark.column)
+        else:
+            msg = msg + """
+We could be wrong, but this one looks like it might be an issue with
+missing quotes.  Always quote template expression brackets when they 
+start a value. For instance:            
+
+    with_items:
+      - {{ foo }}
+
+Should be written as:
+
+    with_items:
+      - "{{ foo }}"      
+
+"""
+            msg = process_common_errors(msg, probline, mark.column)
 
     else:
         # No problem markers means we have to throw a generic
@@ -471,14 +498,22 @@ Note: The error may actually appear before this position: line %s, column %s
     raise errors.AnsibleYAMLValidationFailed(msg)
 
 
-def parse_yaml_from_file(path):
+def parse_yaml_from_file(path, vault_password=None):
     ''' convert a yaml file to a data structure '''
 
+    data = None
+
     try:
-        data = file(path).read()
-        return parse_yaml(data)
+        data = open(path).read()
     except IOError:
-        raise errors.AnsibleError("file not found: %s" % path)
+        raise errors.AnsibleError("file could not read: %s" % path)
+
+    vault = VaultLib(password=vault_password)
+    if vault.is_encrypted(data):
+        data = vault.decrypt(data)
+
+    try:
+        return parse_yaml(data)
     except yaml.YAMLError, exc:
         process_yaml_error(exc, data, path)
 
@@ -600,6 +635,40 @@ def getch():
         termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
     return ch
 
+def sanitize_output(str):
+    ''' strips private info out of a string '''
+
+    private_keys = ['password', 'login_password']
+
+    filter_re = [
+        # filter out things like user:pass@foo/whatever
+        # and http://username:pass@wherever/foo
+        re.compile('^(?P<before>.*:)(?P<password>.*)(?P<after>\@.*)$'),
+    ]
+
+    parts = str.split()
+    output = ''
+    for part in parts:
+        try:
+            (k,v) = part.split('=', 1)
+            if k in private_keys:
+                output += " %s=VALUE_HIDDEN" % k
+            else:
+                found = False
+                for filter in filter_re:
+                    m = filter.match(v)
+                    if m:
+                        d = m.groupdict()
+                        output += " %s=%s" % (k, d['before'] + "********" + d['after'])
+                        found = True
+                        break
+                if not found:
+                    output += " %s" % part
+        except:
+            output += " %s" % part
+
+    return output.strip()
+
 ####################################################################
 # option handling code for /usr/bin/ansible and ansible-playbook
 # below this line
@@ -634,6 +703,12 @@ def base_parser(constants=C, usage="", output_opts=False, runas_opts=False,
         help='use this file to authenticate the connection')
     parser.add_option('-K', '--ask-sudo-pass', default=False, dest='ask_sudo_pass', action='store_true',
         help='ask for sudo password')
+    parser.add_option('--ask-su-pass', default=False, dest='ask_su_pass', action='store_true', 
+        help='ask for su password')
+    parser.add_option('--ask-vault-pass', default=False, dest='ask_vault_pass', action='store_true', 
+        help='ask for vault password')
+    parser.add_option('--vault-password-file', default=None, dest='vault_password_file',
+        help="vault password file")
     parser.add_option('--list-hosts', dest='listhosts', action='store_true',
         help='outputs a list of matching hosts; does not execute anything else')
     parser.add_option('-M', '--module-path', dest='module_path',
@@ -657,11 +732,15 @@ def base_parser(constants=C, usage="", output_opts=False, runas_opts=False,
     if runas_opts:
         parser.add_option("-s", "--sudo", default=constants.DEFAULT_SUDO, action="store_true",
             dest='sudo', help="run operations with sudo (nopasswd)")
-        parser.add_option('-U', '--sudo-user', dest='sudo_user', help='desired sudo user (default=root)',
-            default=None)   # Can't default to root because we need to detect when this option was given
+        parser.add_option('-U', '--sudo-user', dest='sudo_user', default=None,
+                          help='desired sudo user (default=root)')  # Can't default to root because we need to detect when this option was given
         parser.add_option('-u', '--user', default=constants.DEFAULT_REMOTE_USER,
-            dest='remote_user',
-            help='connect as this user (default=%s)' % constants.DEFAULT_REMOTE_USER)
+            dest='remote_user', help='connect as this user (default=%s)' % constants.DEFAULT_REMOTE_USER)
+
+        parser.add_option('-S', '--su', default=constants.DEFAULT_SU,
+                          action='store_true', help='run operations with su')
+        parser.add_option('-R', '--su-user', help='run operations with su as this '
+                                                  'user (default=%s)' % constants.DEFAULT_SU_USER)
 
     if connect_opts:
         parser.add_option('-c', '--connection', dest='connection',
@@ -688,10 +767,36 @@ def base_parser(constants=C, usage="", output_opts=False, runas_opts=False,
 
     return parser
 
-def ask_passwords(ask_pass=False, ask_sudo_pass=False):
+def ask_vault_passwords(ask_vault_pass=False, ask_new_vault_pass=False, confirm_vault=False, confirm_new=False):
+
+    vault_pass = None
+    new_vault_pass = None
+
+    if ask_vault_pass:
+        vault_pass = getpass.getpass(prompt="Vault password: ")
+
+    if ask_vault_pass and confirm_vault:
+        vault_pass2 = getpass.getpass(prompt="Confirm Vault password: ")
+        if vault_pass != vault_pass2:
+            raise errors.AnsibleError("Passwords do not match")
+
+    if ask_new_vault_pass:
+        new_vault_pass = getpass.getpass(prompt="New Vault password: ")
+
+    if ask_new_vault_pass and confirm_new:
+        new_vault_pass2 = getpass.getpass(prompt="Confirm New Vault password: ")
+        if new_vault_pass != new_vault_pass2:
+            raise errors.AnsibleError("Passwords do not match")
+
+    return vault_pass, new_vault_pass
+
+def ask_passwords(ask_pass=False, ask_sudo_pass=False, ask_su_pass=False, ask_vault_pass=False):
     sshpass = None
     sudopass = None
+    su_pass = None
+    vault_pass = None
     sudo_prompt = "sudo password: "
+    su_prompt = "su password: "
 
     if ask_pass:
         sshpass = getpass.getpass(prompt="SSH password: ")
@@ -702,7 +807,13 @@ def ask_passwords(ask_pass=False, ask_sudo_pass=False):
         if ask_pass and sudopass == '':
             sudopass = sshpass
 
-    return (sshpass, sudopass)
+    if ask_su_pass:
+        su_pass = getpass.getpass(prompt=su_prompt)
+
+    if ask_vault_pass:
+        vault_pass = getpass.getpass(prompt="Vault password: ")
+
+    return (sshpass, sudopass, su_pass, vault_pass)
 
 def do_encrypt(result, encrypt, salt_size=None, salt=None):
     if PASSLIB_AVAILABLE:
@@ -756,86 +867,6 @@ def boolean(value):
     else:
         return False
 
-def compile_when_to_only_if(expression):
-    '''
-    when is a shorthand for writing only_if conditionals.  It requires less quoting
-    magic.  only_if is retained for backwards compatibility.
-    '''
-
-    # when: set $variable
-    # when: unset $variable
-    # when: failed $json_result
-    # when: changed $json_result
-    # when: int $x >= $z and $y < 3
-    # when: int $x in $alist
-    # when: float $x > 2 and $y <= $z
-    # when: str $x != $y
-    # when: jinja2_compare asdf  # implies {{ asdf }}
-
-    if type(expression) not in [ str, unicode ]:
-        raise errors.AnsibleError("invalid usage of when_ operator: %s" % expression)
-    tokens = expression.split()
-    if len(tokens) < 2:
-        raise errors.AnsibleError("invalid usage of when_ operator: %s" % expression)
-
-    # when_set / when_unset
-    if tokens[0] in [ 'set', 'unset' ]:
-        tcopy = tokens[1:]
-        for (i,t) in enumerate(tokens[1:]):
-            if t.find("$") != -1:
-                tcopy[i] = "is_%s('''%s''')" % (tokens[0], t)
-            else:
-                tcopy[i] = t
-        return " ".join(tcopy)
-
-    # when_failed / when_changed
-    elif tokens[0] in [ 'failed', 'changed' ]:
-        tcopy = tokens[1:]
-        for (i,t) in enumerate(tokens[1:]):
-            if t.find("$") != -1:
-                tcopy[i] = "is_%s(%s)" % (tokens[0], t)
-            else:
-                tcopy[i] = t
-        return " ".join(tcopy)
-
-    # when_integer / when_float / when_string
-    elif tokens[0] in [ 'integer', 'float', 'string' ]:
-        cast = None
-        if tokens[0] == 'integer':
-            cast = 'int'
-        elif tokens[0] == 'string':
-            cast = 'str'
-        elif tokens[0] == 'float':
-            cast = 'float'
-        tcopy = tokens[1:]
-        for (i,t) in enumerate(tokens[1:]):
-            #if re.search(t, r"^\w"):
-                # bare word will turn into Jinja2 so all the above
-                # casting is really not needed
-                #tcopy[i] = "%s('''%s''')" % (cast, t)
-            t2 = t.strip()
-            if (t2[0].isalpha() or t2[0] == '$') and cast == 'str' and t2 != 'in':
-                tcopy[i] = "'%s'" % (t)
-            else:
-                tcopy[i] = t
-        result = " ".join(tcopy)
-        return result
-
-
-    # when_boolean
-    elif tokens[0] in [ 'bool', 'boolean' ]:
-        tcopy = tokens[1:]
-        for (i, t) in enumerate(tcopy):
-            if t.find("$") != -1:
-                tcopy[i] = "(is_set('''%s''') and '''%s'''.lower() not in ('false', 'no', 'n', 'none', '0', ''))" % (t, t)
-        return " ".join(tcopy)
-
-    # the stock 'when' without qualification (new in 1.2), assumes Jinja2 terms
-    elif tokens[0] == 'jinja2_compare':
-        return " ".join(tokens)
-    else:
-        raise errors.AnsibleError("invalid usage of when_ operator: %s" % expression)
-
 def make_sudo_cmd(sudo_user, executable, cmd):
     """
     helper function for connection plugins to create sudo commands
@@ -855,6 +886,21 @@ def make_sudo_cmd(sudo_user, executable, cmd):
         prompt, sudo_user, executable or '$SHELL', pipes.quote('echo %s; %s' % (success_key, cmd)))
     return ('/bin/sh -c ' + pipes.quote(sudocmd), prompt, success_key)
 
+
+def make_su_cmd(su_user, executable, cmd):
+    """
+    Helper function for connection plugins to create direct su commands
+    """
+    # TODO: work on this function
+    randbits = ''.join(chr(random.randint(ord('a'), ord('z'))) for x in xrange(32))
+    prompt = 'assword: '
+    success_key = 'SUDO-SUCCESS-%s' % randbits
+    sudocmd = '%s %s %s %s -c %s' % (
+        C.DEFAULT_SU_EXE, C.DEFAULT_SU_FLAGS, su_user, executable or '$SHELL',
+        pipes.quote('echo %s; %s' % (success_key, cmd))
+    )
+    return ('/bin/sh -c ' + pipes.quote(sudocmd), prompt, success_key)
+
 _TO_UNICODE_TYPES = (unicode, type(None))
 
 def to_unicode(value):
@@ -900,51 +946,95 @@ def is_list_of_strings(items):
             return False
     return True
 
-def safe_eval(str, locals=None, include_exceptions=False):
+def safe_eval(expr, locals={}, include_exceptions=False):
     '''
     this is intended for allowing things like:
     with_items: a_list_variable
     where Jinja2 would return a string
     but we do not want to allow it to call functions (outside of Jinja2, where
     the env is constrained)
+
+    Based on:
+    http://stackoverflow.com/questions/12523516/using-ast-and-whitelists-to-make-pythons-eval-safe
     '''
-    # FIXME: is there a more native way to do this?
 
-    def is_set(var):
-        return not var.startswith("$") and not '{{' in var
+    # this is the whitelist of AST nodes we are going to 
+    # allow in the evaluation. Any node type other than 
+    # those listed here will raise an exception in our custom
+    # visitor class defined below.
+    SAFE_NODES = set(
+        (
+            ast.Expression,
+            ast.Compare,
+            ast.Str,
+            ast.List,
+            ast.Tuple,
+            ast.Dict,
+            ast.Call,
+            ast.Load,
+            ast.BinOp,
+            ast.UnaryOp,
+            ast.Num,
+            ast.Name,
+            ast.Add,
+            ast.Sub,
+            ast.Mult,
+            ast.Div,
+        )
+    )
 
-    def is_unset(var):
-        return var.startswith("$") or '{{' in var
+    # AST node types were expanded after 2.6
+    if not sys.version.startswith('2.6'):
+        SAFE_NODES.union(
+            set(
+                (ast.Set,)
+            )
+        )
 
-    # do not allow method calls to modules
-    if not isinstance(str, basestring):
+    # builtin functions that are not safe to call
+    INVALID_CALLS = (
+       'classmethod', 'compile', 'delattr', 'eval', 'execfile', 'file',
+       'filter', 'help', 'input', 'object', 'open', 'raw_input', 'reduce',
+       'reload', 'repr', 'setattr', 'staticmethod', 'super', 'type',
+    )
+
+    class CleansingNodeVisitor(ast.NodeVisitor):
+        def generic_visit(self, node):
+            if type(node) not in SAFE_NODES:
+                #raise Exception("invalid expression (%s) type=%s" % (expr, type(node)))
+                raise Exception("invalid expression (%s)" % expr)
+            super(CleansingNodeVisitor, self).generic_visit(node)
+        def visit_Call(self, call):
+            if call.func.id in INVALID_CALLS:
+                raise Exception("invalid function: %s" % call.func.id)
+
+    if not isinstance(expr, basestring):
         # already templated to a datastructure, perhaps?
         if include_exceptions:
-            return (str, None)
-        return str
-    if re.search(r'\w\.\w+\(', str):
-        if include_exceptions:
-            return (str, None)
-        return str
-    # do not allow imports
-    if re.search(r'import \w+', str):
-        if include_exceptions:
-            return (str, None)
-        return str
+            return (expr, None)
+        return expr
+
     try:
-        result = None
-        if not locals:
-            result = eval(str)
-        else:
-            result = eval(str, None, locals)
+        parsed_tree = ast.parse(expr, mode='eval')
+        cnv = CleansingNodeVisitor()
+        cnv.visit(parsed_tree)
+        compiled = compile(parsed_tree, expr, 'eval')
+        result = eval(compiled, {}, locals)
+
         if include_exceptions:
             return (result, None)
         else:
             return result
+    except SyntaxError, e:
+        # special handling for syntax errors, we just return
+        # the expression string back as-is
+        if include_exceptions:
+            return (expr, None)
+        return expr
     except Exception, e:
         if include_exceptions:
-            return (str, e)
-        return str
+            return (expr, e)
+        return expr
 
 
 def listify_lookup_plugin_terms(terms, basedir, inject):
@@ -956,7 +1046,7 @@ def listify_lookup_plugin_terms(terms, basedir, inject):
         #    with_items: {{ alist }}
 
         stripped = terms.strip()
-        if not (stripped.startswith('{') or stripped.startswith('[')) and not stripped.startswith("/"):
+        if not (stripped.startswith('{') or stripped.startswith('[')) and not stripped.startswith("/") and not stripped.startswith('set(['):
             # if not already a list, get ready to evaluate with Jinja2
             # not sure why the "/" is in above code :)
             try:
@@ -979,12 +1069,21 @@ def listify_lookup_plugin_terms(terms, basedir, inject):
 
     return terms
 
-def deprecated(msg, version):
+def deprecated(msg, version, removed=False):
     ''' used to print out a deprecation message.'''
-    if not C.DEPRECATION_WARNINGS:
+
+    if not removed and not C.DEPRECATION_WARNINGS:
         return
-    new_msg = "\n[DEPRECATION WARNING]: %s. This feature will be removed in version %s." % (msg, version)
-    new_msg = new_msg + " Deprecation warnings can be disabled by setting deprecation_warnings=False in ansible.cfg.\n\n"
+
+    if not removed:
+        if version:
+            new_msg = "\n[DEPRECATION WARNING]: %s. This feature will be removed in version %s." % (msg, version)
+        else:
+            new_msg = "\n[DEPRECATION WARNING]: %s. This feature will be removed in a future release." % (msg)
+        new_msg = new_msg + " Deprecation warnings can be disabled by setting deprecation_warnings=False in ansible.cfg.\n\n"
+    else:
+        raise errors.AnsibleError("[DEPRECATED]: %s.  Please update your playbooks." % msg)
+
     wrapped = textwrap.wrap(new_msg, 79)
     new_msg = "\n".join(wrapped) + "\n"
 
